/*


Generates random 2D-paths between start and finish nodes according to densities distribution given by matrix C with leg's contrainsts defined by the adjacency matrix I

P                   = path_generator2(start , finish , I , [C] , [K] , [V] , [delta]);


Inputs
-------

start               Index of starting points (r x 1) where start(i) = 1,...,R, R = 8(Nx-2)(Ny-2) + 10((Nx-2)+(Ny-2)) + 12,
                    where Nx, Ny denote the number of nodes in x and y axis respectivly.
finish              Index of arrival points (q x 1) where finish(i) = 1,...,R
I                   Index of neigbourds (D x R), where D = 8
C                   Probabilities matrix (D x R x L) such that sum(C,1) = 1 (default C is uniform pdf)
K                   Number of path's leg (at least the number of the shortest path, default K = 100)
V                   Number of path's to generate (default V = 1)
delta               Transition legs'angles constraints matrix (D x D), i.e. delta(p_{k}|p_{k-1}) (default delta = ones(D,D));

Ouputs
-------

P                   Path leg's indexes (K x L x V)

Example 1
----------


K                   = 40;
L                   = 1;
V                   = 1;

Nx                  = 40;                
Ny                  = 40;                


x_min               = 0;                 
x_max               = 1;                 
y_min               = 0;                 
y_max               = 1;                 


vect_x              = (x_min : (x_max - x_min)/(Nx - 1) : x_max);
vect_y              = (y_min : (y_max - y_min)/(Ny - 1) : y_max);
[x , y]             = meshgrid(vect_x , vect_y);
XY                  = [x(:)' ; y(:)'];


S                   = Nx*Ny;
start               = floor(S*rand);
finish              = floor(S*rand);
I                   = indice_ij(Nx , Ny);
C                   = mat_uni(I);
C                   = C(: , : , ones(1 , L));
delta               = circulant([1 1 0 0 0 0 0 1 ]);

P                   = path_generator2(start, finish,I, C, K, V, delta);


H                   = reshape(XY(: , P) , [2 , K , L , V]);
plot(XY(1 , :) , XY(2 , :) , '+', squeeze(H(1 , : , :)) , squeeze(H(2 , : , :)) , XY(1 , start) ,  XY(2 , start) , '^' , XY(1 , finish) ,  XY(2 , finish ) , 'sg'  , 'linewidth' , 2 )


Example 2
----------


K                   = 20;
L                   = 1;
V                   = 2;
Q                   = 10;

Nx                  = 10;                
Ny                  = 10;
S                   = Nx*Ny;

start               = floor(S*rand);
finish              = floor(S*rand);



XY                  = rand(2 , S);
[indx , indy]       = find(Kadjacency(XY , Q));
I                   = reshape(indx , Q , S);

C                   = mat_uni(I);
C                   = C(: , : , ones(1 , L));

P                   = path_generator2(start, finish, I, C, K, V);


H                   = reshape(XY(: , P) , [2 , K , L , V]);
plot(XY(1 , :) , XY(2 , :) , '+', squeeze(H(1 , : , :)) , squeeze(H(2 , : , :)) , XY(1 , start) ,  XY(2 , start) , '^' , XY(1 , finish) ,  XY(2 , finish ) , 'sg'  , 'linewidth' , 2 )
text(XY(1 , :),XY(2 , :),num2str((1:S)'))



Example 3
----------


K                   = 60;
L                   = 1;
V                   = 10;

Nx                  = 50;                
Ny                  = 50;                

x_min               = 0;                 
x_max               = 1;                 
y_min               = 0;                 
y_max               = 1;                 


vect_x              = (x_min : (x_max - x_min)/(Nx - 1) : x_max);
vect_y              = (y_min : (y_max - y_min)/(Ny - 1) : y_max);
[x , y]             = meshgrid(vect_x , vect_y);
XY                  = [x(:)' ; y(:)'];


S                   = Nx*Ny;
start               = floor(S*rand);
finish              = floor(S*rand);
I                   = indice_ij(Nx , Ny);
delta               = circulant([1 1 0 0 0 0 0 1 ]);


plot(XY(1 , :) , XY(2 , :) , '+', 'linewidth' , 2 )

title('Select the first exlusion zone', 'fontsize' , 13);
[xx , yy]           = getline(gcf , 'closed');
node                = [xx' ; yy'];

hold on
plot(xx , yy , 'r', 'linewidth' , 2)
[in , bnd]          = inpoly(XY , node);
exclu               = find(in)';
Xexclu              = XY(1,exclu);
Yexclu              = XY(2,exclu);

%bnd                 = convhull(Xexclu, Yexclu);
%plot(Xexclu(bnd) , Yexclu(bnd) , 'ro', 'linewidth' , 2 )
plot(Xexclu , Yexclu , 'ro', 'linewidth' , 2 )

I                   = exclusion(I , exclu);


title('Select the second exlusion zone', 'fontsize' , 13);
[xx , yy]           = getline(gcf , 'closed');
node                = [xx' ; yy'];

hold on
plot(xx , yy , 'r', 'linewidth' , 2)
[in , bnd]          = inpoly(XY , node);
exclu               = find(in)';
Xexclu              = XY(1,exclu);
Yexclu              = XY(2,exclu);
plot(Xexclu , Yexclu , 'ro', 'linewidth' , 2 )

I                   = exclusion(I , exclu);


title('Select start zone', 'fontsize' , 13);

[xx , yy]           = getline(gcf , 'closed');
node                = [xx' ; yy'];
plot(xx , yy , 'k', 'linewidth' , 2)
[in , bnd]          = inpoly(XY , node);
plot(XY(1 , in) , XY(2 , in) , 'y*', 'linewidth' , 2 )
start               = find(in)';


title('Select finish zone', 'fontsize' , 13);

[xx , yy]           = getline(gcf , 'closed');
node                = [xx' ; yy'];
plot(xx , yy , 'k', 'linewidth' , 2)
[in , bnd]          = inpoly(XY , node);
plot(XY(1 , in) , XY(2 , in) , 'm^', 'linewidth' , 2 )
finish              = find(in)';
drawnow


C                   = mat_random(I);
C                   = C(: , : , ones(1 , L));

P                   = path_generator2(start, finish , I , C , K, V, delta);

H                   = reshape(XY(: , P) , [2 , K , L , V]);
plot( squeeze(H(1 , : , :)) , squeeze(H(2 , : , :)) , XY(1 , start) ,  XY(2 , start) , '^' , XY(1 , finish) ,  XY(2 , finish ) , 'sg'  , 'linewidth' , 2 )
hold off
title('Generated paths', 'fontsize' , 13);


To Compile
----------


mex path_generator2.c
 

Author          Sbastien PARIS (sebastien.paris@lis-lab.fr)
------


*/


#include <math.h>
#include <time.h>
#include "mex.h"

#define znew   (z = 36969*(z&65535) + (z>>16) )
#define wnew   (w = 18000*(w&65535) + (w>>16) )
#define MWC    ((znew<<16) + wnew )
#define SHR3   ( jsr ^= (jsr<<17), jsr ^= (jsr>>13), jsr ^= (jsr<<5) )
#define randint SHR3
#define rand() (0.5 + (signed)randint*2.328306e-10)

#ifdef __x86_64__
typedef int UL;
#else
typedef unsigned long UL;
#endif
static UL jsrseed = 31340134 , jsr;

/*--------------------------------------------------------------------------------*/

void randini(void);
void path_generator(double *, double *, double * , double *, int , int  , int ,  double *, int , int, int , int, double *);

/*--------------------------------------------------------------------------------*/

void mexFunction( int nlhs, mxArray *plhs[] , int nrhs, const mxArray *prhs[] )
{
	double *start, *finish, *C,  *I, *delta;
	double *P; 
	double temp;
	const mwIndex  *dimsC; 
	mwIndex  *dimsP;
	int numdimsC, numdimsP = 2;
	int r, q , S, D, K = 100, L = 1, V = 1;	
	int i , j, co, jD;

	/* ------------ Parse INPUT  --------------------- */
	
	if(nrhs < 3)	
	{
		mexErrMsgTxt("At least 3 input argument is required for path_generator");	
	}
	
	
	/* ----- Input 1 ----- */
	
	if(mxIsEmpty(prhs[0]))
	{
	   mexErrMsgTxt("start index is empty, at least one index = 1,...,Nx*Ny");
	}
	if (mxGetN(prhs[0]) != 1)	
	{
		mexErrMsgTxt("start must be (r x 1)");
	}
	start          = mxGetPr(prhs[0]);
	r              = (int)mxGetM(prhs[0]);
	
	/* ----- Input 2 ----- */
	
	if(mxIsEmpty(prhs[1]))
	{
	   mexErrMsgTxt("finish index is empty, at least one index = 1,...,Nx*Ny");
	}
	if (mxGetN(prhs[1]) != 1)
	{
		mexErrMsgTxt("finish must be (q x 1)");
	}
	finish         = mxGetPr(prhs[1]);		
	q              = (int)mxGetM(prhs[1]);

/*
	for(i = 0 ; i < r ; i++)
	{	
		for(j = 0 ; j < q ; j++)	
		{
			if(finish[j] == start[i])
			{
				mexErrMsgTxt("At least one finish index is equal to a start index");	
			}	
		}
	}
	
*/
	
	/* ----- Input 3 ----- */
	
	I              = mxGetPr(prhs[2]);
	D              = mxGetM(prhs[2]);
	S              = mxGetN(prhs[2]);
	
	/* ----- Input 4 ----- */
	
	if(nrhs >= 4)	
	{
		C              = mxGetPr(prhs[3]);
		numdimsC       = mxGetNumberOfDimensions(prhs[3]);
		dimsC          = mxGetDimensions(prhs[3]);
		if ( (dimsC[0] != D) || (dimsC[1] != S))
		{
			mexErrMsgTxt("C must be (D x R)");	
		}
		if(numdimsC > 2)
		{
			L          = dimsC[2];	
		}
	}
	
	else
	{
		C   = (double *)malloc(D*S*sizeof(double));	
		for (j = 0 ; j < S ; j++)
		{
			jD  = j*D;
			co  = 0;
			for(i = 0 ; i < D ; i++)
			{
				if(I[i + jD] != 0.0)  
				{
					co++;	
				}	
			}
			temp = 1.0/co;
			for(i = 0 ; i < D ; i++)
			{
				if(I[i + jD] != 0.0)  
				{
					C[i + jD] = temp;	
				}	
			}	
		}	
	}
	
	/* ----- Input 5 ----- */
	if(nrhs >= 5)	
	{
		K          = (int) mxGetScalar(prhs[4]);	
	}
	/* ----- Input 6 ----- */
	
	if(nrhs >= 6)	
	{
		V          = (int) mxGetScalar(prhs[5]);
		if(V > 1)
		{
			numdimsP = 3;	
		}	
	}
	
	/* ----- Input 7 ----- */
	
	if(nrhs >= 7)	
	{
		delta      =  mxGetPr(prhs[6]);
		if ((mxGetM(prhs[6]) != D) || (mxGetN(prhs[6]) != D)  )	
		{
			mexErrMsgTxt("delta must be (D x D)");		
		}	
	}
	else	
	{
		delta   = (int *)malloc(D*D*sizeof(int));
		for(i = 0 ; i < D*D ; i++)
		{
			delta[i] = 1.0;	
		}		
	}

	/* ----- output 1 ----- */
	
	dimsP       = (mwIndex *)mxMalloc(numdimsP*sizeof(mwIndex));
	dimsP[0]    = K;
	dimsP[1]    = L;

	if(V > 1)	
	{
		dimsP[2]    = V;
	}

	plhs[0]     = mxCreateNumericArray(numdimsP , dimsP , mxDOUBLE_CLASS, mxREAL);	
    P           = mxGetPr(plhs[0]);
		
	/*---------------------------------------------------------------------------------*/
	/* ----------------------- MAIN CALL  -------------------------------------------- */
	/*---------------------------------------------------------------------------------*/
	
	randini();
	path_generator(start, finish, I, C , L, V, K, delta, S , D, r, q, P);
			
	/* ------------ FREE MEMORY  --------------------- */
	
	if(nrhs < 4)
	{
		free(C);
	}
	if(nrhs < 7)	
	{
		free(delta);	
	}
	mxFree(dimsP);
}


/*----------------------------------------------------------------------------------------------*/

void path_generator(double *start, double *finish, double *I , double *C, int L, int V , int K,  double *delta, int S , int D, int r, int q, double *P)					
{
	int i , j , iD  , decision , indI , inddelta , indice , ind_maxi , index2, index3;
	int v , k , kS , SK = S*K, KL=K*L, vKL, l, lK, DS = D*S, lDS, current, index, draw_index;
	bool drawok;
	double  inf = exp(1000), val = -inf , maxi , temp, dump, rr, cP;
	double *J, *current_cost;
	int *arg_J, *current_index;
	
	   /*  Memory Allocation */

	J              = (double *)malloc(SK*sizeof(double));
	current_cost   = (double *)malloc(D*sizeof(double));
	current_index  = (int *)malloc(D*sizeof(int));
	arg_J          = (int *)malloc(SK*sizeof(int));

	for(v = 0 ; v < V ; v++)
	{
		vKL = v*KL;
		for (l = 0 ; l < L ; l++)
		{
			lK  = l*K + vKL;
			lDS = l*DS; 
			for (i = 0 ; i < SK ; i++)
			{
				J[i]                      = val;
				arg_J[i]                  = 0;
			}
			for(i = 0 ; i < r ; i++)
			{
				J[(int) start[i] - 1]     = 1.0;
			}
			for(k = 0 ; k < K - 1 ; k++)
			{
				kS   = k*S;
				for (i = 0 ; i < S ; i++)
				{
					index2   = i + kS + S;
					iD       = i*D;
					current  = 0;
					drawok   = false;
					maxi     = val; 
					temp     = 0.0;
					for (j = 0 ; j < D ; j++)
					{
						index       = j + iD;
						index3      = index + lDS;
						indI        = (int) I[index]; 
						if(indI != 0)
						{
							indice      = indI + kS - 1;
							dump        = J[indice];
							
							if(dump != val)
							{
								decision    = arg_J[indice];
								inddelta    = 1;
								if(decision != 0) /* Add constrainst */
								{
									inddelta    = (int) delta[(decision - 1) + j*D]; 	
								}
								
								if( (inddelta != 0))
								{
									rr                      = C[index3];
									current_cost[current]   = rr;
									current_index[current]  = j;
									dump                   += rr;
/*									dump                   += (1.0 + rand()); */
									temp                   += rr;
									current++;
									drawok                 = true;
									if(dump > maxi)
									{
										maxi                = dump;					
									}				
								}	
							}	
						}	
					}
					if(drawok)
					{
						rr         = rand()*temp;
						cP         = current_cost[0];
						draw_index = 1; 
						while ( (rr > cP) && (draw_index < current) ) 
						{  	
							cP        += current_cost[draw_index];
							draw_index++;	
						}  
						draw_index--;		
						if(maxi != val)	
						{
							J[index2]      = maxi; 
							arg_J[index2]  = current_index[draw_index] + 1;
						}
					}		
				}		
			}
			
			/*---------- Back-Tracking ---------*/
			
			maxi = val;
			kS   = (K-1)*S;
			for (i = 0 ; i < q ; i++)	
			{
				temp = J[(int) finish[i] - 1 + kS];
				if(temp > maxi)
				{
					maxi     = temp;
					ind_maxi = i;	
				}
			}
			temp                 = finish[ind_maxi];
			decision             = arg_J[(int) temp - 1 + kS];
			P[(K-1) + lK]        = temp;
			for (i = K - 2 ; i >= 0 ; i--)	
			{
				temp             = I[decision - 1 + ((int)temp - 1)*D];
				P[i + lK]        = temp;
				decision         = arg_J[(int)temp  - 1 + i*S];			
			}	
		}
	}

	/* Free Memory */

	free(J);
	free(arg_J);
	free(current_index);
	free(current_cost);	
}
/* --------------------------------------------------------------------------- */

 void randini(void) 	 
 {
	 /* SHR3 Seed initialization */
	 jsrseed  = (UL) time( NULL );
	 jsr     ^= jsrseed;	 	 
 }
 /* --------------------------------------------------------------------------- */
